! ------------------------------------------------------------------------------
!  nl.rc
!  This file is part of Unifix BWidget Toolkit
!  Definition of Dutch resources
! ------------------------------------------------------------------------------


! --- symbolic names of buttons ------------------------------------------------

*abortName:   Af&breken
*retryName:   Opnie&uw proberen
*ignoreName:  N&egeren
*okName:      &OK
*cancelName:  &Annuleren
*yesName:     &Ja
*noName:      &Nee


! --- symbolic names of label of SelectFont dialog ----------------------------

*boldName:          Vet
*italicName:        Cursief
*underlineName:     Onderstrepen
*overstrikeName:    Doorhalen
*fontName:          &Lettertype
*sizeName:          &Grootte
*styleName:         &Stijl
*colorPickerName:   &Kleur...


! --- symbolic names of label of PasswdDlg dialog -----------------------------

*loginName:    &Inlognaam
*passwordName: &Wachtwoord


! --- resource for SelectFont dialog ------------------------------------------

*SelectFont.title:	    Lettertypeselectie
*SelectFont.sampletext:	    Voorbeeldtekst


! --- resource for MessageDlg dialog ------------------------------------------

*MessageDlg.noneTitle:      Bericht
*MessageDlg.infoTitle:      Informatie
*MessageDlg.questionTitle:  Vraag
*MessageDlg.warningTitle:   Waarschuwing
*MessageDlg.errorTitle:     Fout

! --- resource for PasswdDlg dialog -------------------------------------------

*PasswdDlg.title:  Voer inlognaam en wachtwoord in

! --- symbolic names of label of SelectColor dialog ----------------------------

*baseColorsName: Base colors
*userColorsName: User colors

