! -----------------------------------------------------------------------------
!  da.rc
!  This file is part of Unifix BWidget Toolkit
!  Definition of Danish resources
! -----------------------------------------------------------------------------


! --- symbolic names of buttons -----------------------------------------------

*abortName:   &Annullér
*retryName:   P&røv igen
*ignoreName:  &Ignorer
*okName:      &OK
*cancelName:  &Cancel
*yesName:     &Ja
*noName:      &Nej


! --- symbolic names of label of SelectFont dialog ----------------------------

*boldName:          Fed
*italicName:        Kursiv
*underlineName:     Understreg
*overstrikeName:    Overstreg
*fontName:          &Font
*sizeName:          &Størrelse
*styleName:         St&il
*colorPickerName:   F&arve...

! --- symbolic names of label of PasswdDlg dialog -----------------------------

*loginName:    &Brugernavn
*passwordName: &Password


! --- resource for SelectFont dialog ------------------------------------------

*SelectFont.title:	    Font-valg
*SelectFont.sampletext:	    Eksempeltekst æøå


! --- resource for MessageDlg dialog ------------------------------------------

*MessageDlg.noneTitle:      Besked
*MessageDlg.infoTitle:      Information
*MessageDlg.questionTitle:  Spørgsmål
*MessageDlg.warningTitle:   Advarsel
*MessageDlg.errorTitle:     Fejl


! --- resource for PasswdDlg dialog -------------------------------------------

*PasswdDlg.title:  Indtast brugernavn og password

! --- symbolic names of label of SelectColor dialog ----------------------------

*baseColorsName: Base colors
*userColorsName: User colors

