! ------------------------------------------------------------------------------
!  pl.rc
!  This file is part of Unifix BWidget Toolkit
!  Definition of english resources
! ------------------------------------------------------------------------------


! --- symbolic names of buttons ------------------------------------------------

*abortName:   &Porzuć
*retryName:   P&onów
*ignoreName:  &Ignoruj
*okName:      &OK
*cancelName:  &Anyluj
*yesName:     &Tak
*noName:      &Nie


! --- symbolic names of label of SelectFont dialog ----------------------------

*boldName:          Pogrubiona
*italicName:        Kursywa
*underlineName:     Podkreślenie
*overstrikeName:    Przekreślenie
*fontName:          &Czcionka:
*sizeName:          &Rozmiar:
*styleName:         St&yl czcionki:
*colorPickerName:   &Kolor...


! --- symbolic names of label of PasswdDlg dialog -----------------------------

*loginName:    &Login
*passwordName: &Hasło


! --- resource for SelectFont dialog ------------------------------------------

*SelectFont.title:	    Wybór czcionki
*SelectFont.sampletext:	    Przykładowy tekst


! --- resource for MessageDlg dialog ------------------------------------------

*MessageDlg.noneTitle:      Wiadomość
*MessageDlg.infoTitle:      Informacja
*MessageDlg.questionTitle:  Pytanie
*MessageDlg.warningTitle:   Ostrzeżenie
*MessageDlg.errorTitle:     Błąd

! --- resource for PasswdDlg dialog -------------------------------------------

*PasswdDlg.title:  Wpisz login i hasło


! --- symbolic names of label of SelectColor dialog ----------------------------

*baseColorsName: Kolory podstawowe
*userColorsName: Kolory niestandardowe

