! ------------------------------------------------------------------------------
!  no.rc
!  This file is part of Unifix BWidget Toolkit
!  Definition of norwegian resources
! ------------------------------------------------------------------------------


! --- symbolic names of buttons ------------------------------------------------

*abortName:   &Om
*retryName:   &Prøv igjen
*ignoreName:  &Ignore
*okName:      &OK
*cancelName:  &Avbryt
*yesName:     &Ja
*noName:      &Nei


! --- symbolic names of label of SelectFont dialog ----------------------------

*boldName:          Halvfet
*italicName:        Kursiv
*underlineName:     Understreking
*overstrikeName:    Overstryke
*fontName:          &Skrift
*sizeName:          &Størrelse
*styleName:         St&il
*colorPickerName:   &Color...


! --- symbolic names of label of PasswdDlg dialog -----------------------------

*loginName:    &Logg inn
*passwordName: &Passord


! --- resource for SelectFont dialog ------------------------------------------

*SelectFont.title:	    Skriftvalg
*SelectFont.sampletext:	    Prøve tekst


! --- resource for MessageDlg dialog ------------------------------------------

*MessageDlg.noneTitle:      Melding
*MessageDlg.infoTitle:      Informasjon
*MessageDlg.questionTitle:  Spørsmål
*MessageDlg.warningTitle:   Advarsel
*MessageDlg.errorTitle:     Feil

! --- resource for PasswdDlg dialog -------------------------------------------

*PasswdDlg.title:  Skriv inn logginn og passord

! --- symbolic names of label of SelectColor dialog ----------------------------

*baseColorsName: Base colors
*userColorsName: User colors

