! ------------------------------------------------------------------------------
!  hu.rc
!  This file is part of Unifix BWidget Toolkit
!  Definition of english resources
! ------------------------------------------------------------------------------


! --- symbolic names of buttons ------------------------------------------------

*abortName:   &Megszakítás
*retryName:   Új&ra
*ignoreName:  &Kihagyás
*okName:      &OK
*cancelName:  Még&sem
*yesName:     &Igen
*noName:      &Nem


! --- symbolic names of label of SelectFont dialog ----------------------------

*boldName:       Félkövér
*italicName:     Dőlt
*underlineName:  Aláhúzott
*overstrikeName: Felülírás
*fontName:       &Betűtípus
*sizeName:       &Méret
*styleName:      S&tílus
*colorPickerName:   &Szín...


! --- symbolic names of label of PasswdDlg dialog -----------------------------

*loginName:    &Felhasználónév
*passwordName: &Jelszó


! --- resource for SelectFont dialog ------------------------------------------

*SelectFont.title:	    Betűtípus kiválasztása
*SelectFont.sampletext:	    Példaszöveg


! --- resource for MessageDlg dialog ------------------------------------------

*MessageDlg.noneTitle:      Üzenet
*MessageDlg.infoTitle:      Információ
*MessageDlg.questionTitle:  Kérdés
*MessageDlg.warningTitle:   Figyelmeztetés
*MessageDlg.errorTitle:     Hiba

! --- resource for PasswdDlg dialog -------------------------------------------

*PasswdDlg.title:  Add meg a felhasználónevet és a jelszót


! --- symbolic names of label of SelectColor dialog ----------------------------

*baseColorsName: Alapszínek
*userColorsName: Felhasználói színek

